import tkinter as tk
from PIL import Image, ImageTk
import os
from .tkinterdnd2 import TkinterDnD, DND_FILES, COPY

from tkinter.scrolledtext import ScrolledText
from pystegy import stegyImage

SRC_FILE = 'file'
SRC_RANDOM = 'random'
IMAGE_SIZE = (320,240)
OP_IMAGE_SIZE = (160,120)

class FrameButtons(tk.Frame):
  
    def __init__(self, master, content, cnf={}, **kw):
        tk.Frame.__init__(self, master, cnf, **kw)# self.configure(/ackground = FRAME_COLOR)
        options = {'padx': 10, 'pady': 2}
        self.content = content

        self.buttons = {}
        # button
        self.buttonOk = tk.Button(self, text='Go !')
        self.buttonOk['command'] = master.generate
        # self.buttonOk.grid(row=0, column=3, sticky = 'EN', **options)
        self.buttonOk.pack(side='right', **options)
        self.file_generated = ""

        # placeholderlabel
        self.message = tk.Label(self, text='')
        # self.message.grid(row=0, column=0, columnspan=3, padx = 560)
        self.message.pack(side='bottom', fill='x', expand=True,  padx=320)


        # options button
        self.buttons['src'] = tk.Button(self, text='Options...', relief="groove")
        self.buttons['src']['command'] = lambda : self.showFrame("src") 
        # self.buttons['src'].grid(row=0, column=1, **options)
        self.buttons['src'].pack(side='left', **options)
        # key button
        self.buttons['key'] = tk.Button(self, text='Clé...', relief="groove")
        self.buttons['key']['command'] = lambda : self.showFrame("key") 
        # self.buttons['key'].grid(row=0, column=2, sticky = 'W', **options)
        self.buttons['key'].pack(side='left', **options)

        self.showFrame("mess")
        
    def showFrame(self, frName, force = False):
        for n, b in self.buttons.items():
            b.configure(relief = 'groove')
        if self.content.showFrame(frName, force) == True:
            button = self.buttons.get(frName)
            if not(button is None):
                button.configure(relief = 'sunken')



class FrameMessage(tk.LabelFrame):
  
    def __init__(self, master=None, cnf={}, **kw):
        tk.LabelFrame.__init__(self, master, cnf, **kw)# self.configure(/ackground = FRAME_COLOR)
        options = {'padx': 10, 'pady': 10}
 
        # entry
        self.inputValue = tk.StringVar()
        self.inputValue.set("entrez votre message ici")
        self.text_area = ScrolledText(self, wrap=tk.WORD,
                                      width=40, height=10)
                                    #   font=("Times New Roman", 15))
        # self.input = tk.Entry(self, textvariable = self.inputValue)
        # self.text_area.insert(tk.INSERT,self.op['txt'].get())

        # s0elf.text_area.grid(row=0, column=1, columnspan = 3, **options)
        self.text_area.pack(side='top', fill='both', expand = True,**options)
        self.text_area.focus_set()

    def message(self, text = None):
        if text is None:
            return self.text_area.get("1.0",'end-1c')
        else:
            self.text_area.delete(1.0, 'end')
            self.text_area.insert(1.0,text)
  


class FrameOptionKey(tk.LabelFrame):
  
    def __init__(self, master=None, cnf={}, **kw):
        tk.LabelFrame.__init__(self, master, cnf, **kw)# self.configure(/ackground = FRAME_COLOR)
        options = {'padx': 10, 'pady': 10}
 
        # options variables
        self.op = {}
        self.op['key'] = tk.StringVar()
        self.op['key'].set("")
        # key : textbox
        self.opKey = tk.Entry(self, width=55, textvariable = self.op['key'])
        self.opKey.pack(side='top', fill='x', expand=True, **options)
        self.opKey.focus_set()

    def key(self, key = None):
        if key is None:
            return self.op['key'].get()
        else:
            self.op['key'].set(key)


    def options(self, key = None, **op):
        opts = {}
        
        if (key is None):
            # fill opts
            opts['key'] = self.op['key'].get() 
            return opts

        if len(key):
            self.op['key'].set(key)  

        return opts



class FrameOptionSource(tk.LabelFrame):

    def __init__(self, master=None, cnf={}, **kw):
        tk.LabelFrame.__init__(self, master, cnf, **kw)# self.configure(/ackground = FRAME_COLOR)
        options = {'padx': 10, 'pady': 10}
 
        # options variables
        self.op = {}
        self.op['src'] = tk.StringVar()
        self.op['src'].set(SRC_RANDOM)
        self.op['topics'] = tk.StringVar()
        self.op['topics'].set("dog,cat")
        

        # topic : radio
        self.r1 = tk.Radiobutton(self, text = "Aléatoire", variable = self.op['src'], value = SRC_RANDOM)
        self.r1.grid(row=0, column= 0, sticky = 'W')
        # image : radio
        self.r2 = tk.Radiobutton(self, text = "Image", variable = self.op['src'], value = SRC_FILE)
        self.r2.grid(row=2, column= 0, sticky = 'W')
        
        # topic : label        
        tk.Label(self, text='''image prise aléatoirement sur Flicker, vous pouvez 
spécifier un ou plusieurs topics :''', anchor='w').grid(row=0, column= 1)
        # topic : textbox
        self.opTopics = tk.Entry(self, textvariable = self.op['topics'])
        self.opTopics.grid(row=1, column= 1, sticky='EW')

        # image : label
        tk.Label(self, text='''image de votre choix, droppez-la ici''', anchor='w').grid(row=2, column= 1, sticky='EW')
        # image : image
        global opPhoto
        opIimageSrc = Image.open( os.path.join('ress','src.png')).resize(OP_IMAGE_SIZE, Image.BILINEAR)
        opPhoto = ImageTk.PhotoImage(opIimageSrc)
        self.opSrc = tk.Canvas(self, width = OP_IMAGE_SIZE[0], height = OP_IMAGE_SIZE[1], cursor="hand2")
        self.opSrcImage_in_canvas = self.opSrc.create_image(0,0, image=opPhoto, anchor=tk.NW)
        self.opSrc.grid(row=3, column=1, sticky= tk.NS, **options)
        # DnD stuff
        self.opSrc.drop_target_register(DND_FILES)
        self.opSrc.dnd_bind('<<Drop>>', self.generateSrc)

        self.opTopics.focus_set()
    
    def generateSrc(self, event):
        filepath = event.data
        global opPhoto
        
        self.r2.select()
        op = {}
        op['src'] = filepath 
        op['dest'] = 'ress/src.png' 
        result = stegyImage().encode( "", **op)
        # update canvas image with result
        image = Image.open( result)
        image = image.resize(OP_IMAGE_SIZE)
        opPhoto = ImageTk.PhotoImage(image) 
        self.opSrc.itemconfigure(self.opSrcImage_in_canvas, image = opPhoto)


    def options(self, src : str = None, topics : str = None, **op):
        opts = {}
        if (src is None) & (topics is None) :
            # fill opts
            val = self.op['src'].get()
            if len(val) > 0:
                opts['src'] =  val
            opts['topics'] = self.op['topics'].get() 
            return opts

        if len(src):        
            if src == SRC_RANDOM:
                self.r1.select()
            else:
                self.r2.select()

        if len(topics):
            self.op['topics'].set(topics)

        return opts

    def optionsSelected(self):
        opts = {}
        # if not(options is None):
        #     opts = options

        if self.op['src'].get() == SRC_RANDOM:
            opts['topics'] = self.op['topics'].get() 
        else:
            opts['src'] = 'ress/src.png' 

        return opts

        



import os
import tkinter as tk
from tkinter import font

from tkinter.messagebox import askyesno


class TitleBar(tk.Frame):
    PATH_LOCK = 'TitleBar.lck'

    """Title Bar widget

    """    
   

    def __init__(self, master, title="TitleBar", buttons=None, font= None, **options):
        """TitleBar constructon with given master widget
        a buttons descritor and classical options ( padx=, padx=, ...)

        Args:
            master (tk.Widget): master widget
            buttons ([dict], optional): dictionary   { "X" : fn1, "Y" : fn2}
               create Button(... title="X", command=fn1 ...) and Button(... title="Y", command=fn2 ...) . 
               Defaults to None.
        """        
        tk.Frame.__init__(self, master)
        
        self._default = {}
        self.designButton = None
        
        self.configure(**options)
        # save given options like padx, padx...
        for txt, opts in options.items():
            self._default[txt] = opts

        self.isDesignMode = True
        self.bHideSystemBar = False
        # are we in design mode ?
        self.checkDesignMode()
        # init label with app title and add buttons
        self.label = tk.Label(self, text=title, **self._default, anchor='w')
        self.label.pack(expand=True, fill="x", side='left', padx = 5)

        self._init_buttons(buttons)

        # set D'nD callbacks
        self.label.bind('<B1-Motion>', self.move_window)
        self.label.bind('<ButtonPress-1>', self.move_window_init)
        self.label.bind('<ButtonRelease-1>', self.move_window_end)
        self.label.bind('<ButtonPress-3>', self.userLock)

    
    def userLock(self, event):
        if not(os.path.exists(self.PATH_LOCK)):
            if askyesno("verrouillage", '''Verouiller la geomètrie de l'application  
            (Effacer le fichier "'''+self.PATH_LOCK+'''" annulera cette action'''):
                with open(self.PATH_LOCK, "wt") as f:
                    f.write(self.winfo_toplevel().geometry())

                self.checkDesignMode()
 
    def checkDesignMode(self):
        self.isDesignMode = True
        try:
            if os.path.exists(self.PATH_LOCK):
                with open(self.PATH_LOCK, "rt") as f:
                    geom = f.read()
                    if len(geom) > 0 :
                        self.isDesignMode = False
                        self.hideRealTitle(geom)
                        if self.designButton:
                            self.designButton.forget()
        finally:
            pass

        return self.isDesignMode



    # 
    #  Drag'n Drop => Move APP on screen
    # 

    def move_window_init(self, event):
        # only when there is no system Title bar
        if self.bHideSystemBar:
            self.delta_x = event.x
            self.delta_y = event.y
        
            self.configure(cursor="hand2")

    def move_window_end(self, event):
        # only when there is no system Title bar
        if self.bHideSystemBar:
            self.configure(cursor="")

    def move_window(self, event):
        # only when there is no system Title bar
        if self.bHideSystemBar:
            self.winfo_toplevel().geometry('+{0}+{1}'.format(event.x_root-self.delta_x, event.y_root-self.delta_y))

    # initialise UI

    def _init_buttons(self, buttons: dict):
        """create buttons and install its highlights callback
            use the 'buttons' dict from constructor
            { "X" : fn1, "Y" : fn2}
            create Button(... title="X", command=fn1 ...) and Button(... title="Y", command=fn2 ...) 

        Args:
            buttons (dict): label , command dict
        """        
        for txt, cmd in reversed(buttons.items()):
            b=tk.Button(self, text=txt, command = cmd, **self._default)
            b.pack(side="right", padx=2, pady = 2)
            b.bind('<Enter>', lambda evt : self._setBgColor(evt, 'red'))
            b.bind('<Leave>', lambda evt : self._setBgColor(evt, None))
            
        if self.isDesignMode:
            self.designButton = tk.Button(self, text='~', command=self.toggleRealTitle, **self._default)
            self.designButton.pack(side="right", padx=2, pady = 2)
            self.designButton.bind('<Enter>', lambda evt : self._setBgColor(evt, 'red'))
            self.designButton.bind('<Leave>', lambda evt : self._setBgColor(evt, None))

    # callback to highlight toolbar buttons on mouse over event
    def _setBgColor(self, event, color = None):
        if color == None:
            event.widget['bg'] = event.widget.master['bg']
            event.widget['fg'] = 'black'
        else:
            event.widget['bg'] = color
            event.widget['fg'] = 'white'    

    def toggleRealTitle(self):
        """toogle system title bar visibily and deactivate geometry calculus
        """        
        if self.isDesignMode:
            self.bHideSystemBar = not(self.bHideSystemBar)
            self.winfo_toplevel().overrideredirect(self.bHideSystemBar) # turns off title bar, geometry

    def hideRealTitle(self, geometry):
        """hide system title bar and set window size and position
        """        
        self.bHideSystemBar = True
        self.winfo_toplevel().overrideredirect(self.bHideSystemBar) # turns off title bar, geometry
        self.winfo_toplevel().geometry(geometry)




import os
import json
from datetime import datetime

import tkinter as tk
from tkinter.messagebox import *
from .tkinterdnd2 import TkinterDnD, DND_FILES, COPY
from PIL import Image, ImageTk

from .frame_option import FrameOptionSource,FrameOptionKey, FrameMessage, FrameButtons
from .ui.title import TitleBar

from pystegy import stegyImage
from pystegy.utils import BadKey, NoMessage

import pkg_resources
import shutil

consts = {"FRAME_COLOR" : 'gray30'}

FRAME_COLOR = 'gray30'
IMAGE_SIZE = (320,240)
OP_IMAGE_SIZE = (97,80)

class ContentFrame(tk.Frame):
    def __init__(self, container):
        super().__init__(container)

        self.FRAMES = {}
        
        self.FRAMES["src"] = FrameOptionSource(self, text= 'image')
        self.FRAMES["key"] = FrameOptionKey(self, text= 'clé')
        self.FRAMES["mess"] = FrameMessage(self, text= 'message')
        
        self.current = "src"
        self.showFrame('src')

    def showFrame(self, name, force = False):
        ret = False

        if force == False :
            if name == self.current == 'key':
                name = 'mess'
            elif name == self.current == 'src':
                name = 'mess'
            else:
                ret = True
        else:
            ret = True

        self.current = name

        self.FRAMES["src"].pack_forget()
        self.FRAMES["key"].pack_forget()
        self.FRAMES["mess"].pack_forget()

        self.FRAMES[name].pack(side ='left', fill = 'both', expand=True)

        return ret

    def message(self, text = None):
        return self.FRAMES["mess"].message( text)

    def key(self):
        op = self.FRAMES["key"].options()
        return op['key']

    def options(self, **op):
        opts = {}

        opts.update(self.FRAMES["src"].options(**op))
        opts.update(self.FRAMES["key"].options(**op))

        return opts

    def optionsSelected(self):
        opts = {}

        opts.update(self.FRAMES["src"].optionsSelected())
        opts.update(self.FRAMES["key"].options())

        return opts

    def ReadConf(self):
        try :
            with open('stegy.cnf', 'rt') as conf:
                op = json.load(conf)
                self.options(**op)
        except:
            pass

    def WriteConf(self):
        try :
            with open('stegy.cnf', 'wt') as conf:
                op = self.options()
                json.dump(op, conf)
        except:
            pass


class TitleFrame(tk.Frame):
    def __init__(self, container):
        super().__init__(container)
        
        # self.configure(background = consts['FRAME_COLOR'])
        toptions = {'padx' : 0, 
                    'pady': 0, 
                    'relief':'flat', 
                    'bg' : consts['FRAME_COLOR'],
                    'font' : ('Times New Roman' , 10)
                    }

        self.title = TitleBar(self, title = "Stegy", buttons={"X": container.destroy},  **toptions)
        self.title.grid(row = 0, column=0, columnspan = 2, sticky="WE")

        options = {'padx' : 5, 
                    'pady': 5
                    }

        # canvas
        global photo
        imageDef = Image.open( os.path.join('ress','stegy.bmp'))
        photo = ImageTk.PhotoImage(imageDef) 
        self.imCan = tk.Canvas(self, width = IMAGE_SIZE[0], height = IMAGE_SIZE[1], cursor="hand2")
        self.image_in_canvas = self.imCan.create_image(0,0, image=photo, anchor=tk.NW)
        self.imCan.grid(row=1, column=0 , sticky='W', **options)
        # Dnd stuff
        self.imCan.drop_target_register(DND_FILES)
        self.imCan.dnd_bind('<<Drop>>', self.fileDroppedOnMe)
        self.imCan.drag_source_register(DND_FILES)
        self.imCan.dnd_bind('<<DragInitCmd>>', self.dragResult)
        # self.share.drag_source_register(DND_FILES)
        # self.share.dnd_bind('<<DragInitCmd>>', self.resultDragInit)





        self.content = ContentFrame(self)
        self.content.grid(row = 1, column=1, sticky="WE", **options)

        self.buttons = FrameButtons(self, self.content)

        self.buttons.grid(row = 2, column = 0, columnspan = 2, **options)
        self.file_generated = None
        self.stegy = stegyImage()


        self.content.ReadConf()

    def generate(self):
        self.content.WriteConf()
        
        op = self.content.optionsSelected()
        op['dest'] = self.genDestFileName()
        
        self.file_generated = os.path.abspath(
                self.stegy.encode(self.content.message(),  **op ))

        self.verify(self.file_generated)

    def genDestFileName(self):

        now = datetime.now() # current date and time
        return now.strftime("data/%H_%M_%S.png")


    def verify(self, filepath):
        global photo2
        
        # clean results info 
        # self.message.config( text="")
        # self.share['state'] = 'disabled'
        
        # update canvas image with result
        image = Image.open( filepath)
        photo2 = ImageTk.PhotoImage(image) 
        self.imCan.itemconfigure(self.image_in_canvas, image = photo2)
        
        try:    
            # re-extract text to verify validity
            txt = self.stegy.decode(filepath, key = self.content.key())
            # update label accordingly
            self.content.message(txt)
            self.buttons.showFrame("mess", True)
            if len(txt) > 50:
                showinfo(title='message', message=txt)
        except BadKey:
            showerror(message = '''Erreur de décodage du message !
            Essayez avec une autre clé de décodage''')
            self.buttons.showFrame("key", True)
        except NoMessage:
            showinfo(message = 'désolé pas de message dans cette image !')
       


    def fileDroppedOnMe(self, event):
        filepath = event.data
        self.verify(filepath)

    def dragResult(self, event):
        #   source : https://wiki.tcl-lang.org/page/TkDND+Tutorial   
        # bind .drag_source_text <<DragInitCmd>> \
        #   {list copy DND_Text {Some nice dropped text!}}
        # bind .drag_source_files <<DragInitCmd>> \
        #   {list {copy move} DND_Files [list $filename $filename]}

        return (COPY,DND_FILES, (self.file_generated,) )



    def getContent(self):
        return self.content

    def getTitle(self):
        return self.title




class StegyApp(TkinterDnD.Tk):


    def __init__(self):
        super().__init__()

        unpackResources()
    
        # configure the root window
        self.title('Stegy')

        self.iconbitmap('./ress/stegy.ico')

        self.initDirs()

        self.frame = TitleFrame(self)
        self.frame.grid(row = 0, column=0, sticky="WESN")


    def initDirs(self):
        if not(os.path.exists('./ress')):
            os.makedirs('./ress')
            # // appeler setegy pour creer un src.png
        if not(os.path.exists('ress\src.png')):
            s = stegyImage()
            s.encode("", topics='dinosaure', dest='ress\src.png' )
        if not(os.path.exists('data')):
            os.mkdir('data')
            
def unpackResources():
    locpath = ('./ress', './data')
    for l in locpath:
        if not (os.path.exists(os.path.abspath(l))):
            os.makedirs(os.path.abspath(l))

    lst = {'datas/stegy.ico' : 'ress/stegy.ico',
            'datas/share32.png' : 'ress/share32.png',
            'datas/stegy.bmp' : 'ress/stegy.bmp',
            'datas/src.png' : 'ress/src.png',
            'datas/TitleBar.lck' : 'TitleBar.lck',
            'datas/stegy.cnf' : 'stegy.cnf' }

    for source,dest  in lst.items():
        src = pkg_resources.resource_filename(__name__, source)
        if not(os.path.exists(dest)) :
            shutil.copyfile(src, dest)
        


if __name__ == "__main__":
    app = StegyApp()
    app.mainloop()






import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pystegy_ui", # Replace with your own username
    version="0.0.2",
    author="d.char",
    author_email="d.charentus@gmail.com",
    description="small steganography utility tkinter interface",
    long_description=long_description,
    long_description_content_type="text/markdown",
    # url="https://github.com/pypa/sampleproject",
    # project_urls={
    #     "Bug Tracker": "https://github.com/pypa/sampleproject/issues",
    # },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    # package_dir={"": "stegy_ui"},
    # packages=setuptools.find_packages(where="pystegy"),
    # py_modules=['stegyImage'],
    packages=["stegy_ui", "stegy_ui.tkinterdnd2", "stegy_ui.ui"],
    include_package_data=True,
    # package_data={
    #     # If any package contains *.txt files, include them:
    #     "": ["*.txt"],
    #     # And include any *.dat files found in the "data" subdirectory
    #     # of the "mypkg" package, also:
    #     "stegy_ui": ["TitleBar.lck", "data/result.png", "ress/src.png", "ress/stegy.ico", "ress/stegy.bmp", "ress/share32.png"],
    # },
    install_requires=[
          "pystegy>=0.0.2"
      ],
    python_requires=">=3.6",
)
